@@ -41,7 +41,7 @@
 #
 ##############################################################################
 #############################################################################
-#Copyright (c) 2010 - 2016, Jo Bovy
+#Copyright (c) 2010 - 2018, Jo Bovy
 #All rights reserved.
 #
 #Redistribution and use in source and binary forms, with or without 
@@ -72,6 +72,7 @@ from functools import wraps
 import math as m
 import numpy as nu
 import scipy as sc
+from galpy.util import _rotate_to_arbitrary_vector
 from galpy.util.config import __config__
 _APY_COORDS= __config__.getboolean('astropy','astropy-coords')
 _APY_LOADED= True
@@ -82,7 +83,22 @@ except ImportError:
     _APY_LOADED= False
 _APY_COORDS*= _APY_LOADED
 _DEGTORAD= m.pi/180.
-_K=4.74047
+if _APY_LOADED:
+    _K= (1.*units.mas/units.yr).to(units.km/units.s/units.kpc,
+                                   equivalencies=units.dimensionless_angles())\
+                                   .value
+else:
+    _K=4.74047
+# numpy 1.14 einsum bug causes astropy conversions to fail in py2.7 -> turn off
+if _APY_COORDS:
+    ra, dec= nu.array([192.25*_DEGTORAD]), nu.array([27.4*_DEGTORAD])
+    c= apycoords.SkyCoord(ra*units.rad,dec*units.rad,
+                          equinox='B1950',frame='fk4')
+    # This conversion fails bc of einsum bug
+    try:
+        c= c.transform_to(apycoords.Galactic)
+    except TypeError: # pragma: no cover
+        _APY_COORDS= False
 def scalarDecorator(func):
     """Decorator to return scalar outputs as a set"""
     @wraps(func)
@@ -176,6 +192,8 @@ def radec_to_lb(ra,dec,degree=False,epoch=2000.0):
                    nu.cos(dec)*nu.sin(ra),
                    nu.sin(dec)])
     galXYZ= nu.dot(T,XYZ)
+    galXYZ[2][galXYZ[2] > 1.]= 1.
+    galXYZ[2][galXYZ[2] < -1.]= -1.
     b= nu.arcsin(galXYZ[2])
     l= nu.arctan2(galXYZ[1]/sc.cos(b),galXYZ[0]/sc.cos(b))
     l[l<0.]+= 2.*nu.pi
@@ -861,7 +879,7 @@ def cov_dvrpmllbb_to_vxyz_single(d,e_d,e_vr,pmll,pmbb,cov_pmllbb,l,b):
     return sc.dot(R.T,sc.dot(cov_vrvlvb,R))
 
 @scalarDecorator
-def XYZ_to_galcenrect(X,Y,Z,Xsun=1.,Zsun=0.):
+def XYZ_to_galcenrect(X,Y,Z,Xsun=1.,Zsun=0.,_extra_rot=True):
     """
     NAME:
 
@@ -883,6 +901,8 @@ def XYZ_to_galcenrect(X,Y,Z,Xsun=1.,Zsun=0.):
        
        Zsun - Sun's height above the midplane
 
+       _extra_rot= (True) if True, perform an extra tiny rotation to align the Galactocentric coordinate frame with astropy's definition
+
     OUTPUT:
 
        (Xg, Yg, Zg)
@@ -893,7 +913,11 @@ def XYZ_to_galcenrect(X,Y,Z,Xsun=1.,Zsun=0.):
 
        2016-05-12 - Edited to properly take into account the Sun's vertical position; dropped Ysun keyword - Bovy (UofT)
 
+       2018-04-18 - Tweaked to be consistent with astropy's Galactocentric frame - Bovy (UofT)
+
     """
+    if _extra_rot:
+        X,Y,Z= nu.dot(galcen_extra_rot,nu.array([X,Y,Z]))
     dgc= nu.sqrt(Xsun**2.+Zsun**2.)
     costheta, sintheta= Xsun/dgc, Zsun/dgc
     return nu.dot(nu.array([[costheta,0.,-sintheta],
@@ -902,7 +926,7 @@ def XYZ_to_galcenrect(X,Y,Z,Xsun=1.,Zsun=0.):
                   nu.array([-X+dgc,Y,nu.sign(Xsun)*Z])).T
 
 @scalarDecorator
-def galcenrect_to_XYZ(X,Y,Z,Xsun=1.,Zsun=0.):
+def galcenrect_to_XYZ(X,Y,Z,Xsun=1.,Zsun=0.,_extra_rot=True):
     """
     NAME:
 
@@ -916,9 +940,11 @@ def galcenrect_to_XYZ(X,Y,Z,Xsun=1.,Zsun=0.):
 
        X, Y, Z - Galactocentric rectangular coordinates
 
-       Xsun - cylindrical distance to the GC
+       Xsun - cylindrical distance to the GC (can be array of same length as X)
        
-       Zsun - Sun's height above the midplane
+       Zsun - Sun's height above the midplane (can be array of same length as X)
+
+       _extra_rot= (True) if True, perform an extra tiny rotation to align the Galactocentric coordinate frame with astropy's definition
 
     OUTPUT:
 
@@ -930,14 +956,32 @@ def galcenrect_to_XYZ(X,Y,Z,Xsun=1.,Zsun=0.):
 
        2016-05-12 - Edited to properly take into account the Sun's vertical position; dropped Ysun keyword - Bovy (UofT)
 
+       2017-10-24 - Allowed Xsun/Zsun to be arrays - Bovy (UofT)
+
+       2018-04-18 - Tweaked to be consistent with astropy's Galactocentric frame - Bovy (UofT)
+
     """
     dgc= nu.sqrt(Xsun**2.+Zsun**2.)
     costheta, sintheta= Xsun/dgc, Zsun/dgc
-    return nu.dot(nu.array([[-costheta,0.,-sintheta],
-                            [0.,1.,0.],
-                            [-nu.sign(Xsun)*sintheta,0.,
-                              nu.sign(Xsun)*costheta]]),
-                  nu.array([X,Y,Z])).T+nu.array([dgc,0.,0.])
+    if isinstance(Xsun,nu.ndarray):
+        zero= nu.zeros(len(Xsun))
+        one= nu.ones(len(Xsun))
+        Carr= nu.rollaxis(nu.array([[-costheta,zero,-sintheta],
+                                    [zero,one,zero],
+                                    [-nu.sign(Xsun)*sintheta,zero,
+                                      nu.sign(Xsun)*costheta]]),2)
+        out= ((Carr*nu.array([[X,X,X],[Y,Y,Y],[Z,Z,Z]]).T).sum(-1)
+                 +nu.array([dgc,zero,zero]).T)
+    else:
+        out= nu.dot(nu.array([[-costheta,0.,-sintheta],
+                              [0.,1.,0.],
+                              [-nu.sign(Xsun)*sintheta,0.,
+                                nu.sign(Xsun)*costheta]]),
+                    nu.array([X,Y,Z])).T+nu.array([dgc,0.,0.])
+    if _extra_rot:
+        return nu.dot(galcen_extra_rot.T,out.T).T    
+    else:
+        return out
 
 def rect_to_cyl(X,Y,Z):
     """
@@ -1047,9 +1091,8 @@ def spher_to_cyl(r, theta, phi):
     z = r*nu.cos(theta)
     return (R,z, phi)
 
-
 @scalarDecorator
-def XYZ_to_galcencyl(X,Y,Z,Xsun=1.,Zsun=0.):
+def XYZ_to_galcencyl(X,Y,Z,Xsun=1.,Zsun=0.,_extra_rot=True):
     """
     NAME:
 
@@ -1071,6 +1114,8 @@ def XYZ_to_galcencyl(X,Y,Z,Xsun=1.,Zsun=0.):
        
        Zsun - Sun's height above the midplane
 
+       _extra_rot= (True) if True, perform an extra tiny rotation to align the Galactocentric coordinate frame with astropy's definition
+
     OUTPUT:
 
        R,phi,z
@@ -1080,11 +1125,12 @@ def XYZ_to_galcencyl(X,Y,Z,Xsun=1.,Zsun=0.):
        2010-09-24 - Written - Bovy (NYU)
 
     """
-    XYZ= nu.atleast_2d(XYZ_to_galcenrect(X,Y,Z,Xsun=Xsun,Zsun=Zsun))
+    XYZ= nu.atleast_2d(XYZ_to_galcenrect(X,Y,Z,Xsun=Xsun,Zsun=Zsun,
+                                         _extra_rot=_extra_rot))
     return nu.array(rect_to_cyl(XYZ[:,0],XYZ[:,1],XYZ[:,2])).T
     
 @scalarDecorator
-def galcencyl_to_XYZ(R,phi,Z,Xsun=1.,Zsun=0.):
+def galcencyl_to_XYZ(R,phi,Z,Xsun=1.,Zsun=0.,_extra_rot=True):
     """
     NAME:
 
@@ -1098,9 +1144,11 @@ def galcencyl_to_XYZ(R,phi,Z,Xsun=1.,Zsun=0.):
 
        R, phi, Z - Galactocentric cylindrical coordinates
 
-       Xsun - cylindrical distance to the GC
+       Xsun - cylindrical distance to the GC (can be array of same length as R)
        
-       Zsun - Sun's height above the midplane
+       Zsun - Sun's height above the midplane (can be array of same length as R)
+
+       _extra_rot= (True) if True, perform an extra tiny rotation to align the Galactocentric coordinate frame with astropy's definition
 
     OUTPUT:
 
@@ -1110,12 +1158,16 @@ def galcencyl_to_XYZ(R,phi,Z,Xsun=1.,Zsun=0.):
 
        2011-02-23 - Written - Bovy (NYU)
 
+       2017-10-24 - Allowed Xsun/Zsun to be arrays - Bovy (UofT)
+
     """
     Xr,Yr,Zr= cyl_to_rect(R,phi,Z)
-    return galcenrect_to_XYZ(Xr,Yr,Zr,Xsun=Xsun,Zsun=Zsun)
+    return galcenrect_to_XYZ(Xr,Yr,Zr,Xsun=Xsun,Zsun=Zsun,
+                             _extra_rot=_extra_rot)
     
 @scalarDecorator
-def vxvyvz_to_galcenrect(vx,vy,vz,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
+def vxvyvz_to_galcenrect(vx,vy,vz,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.,
+                         _extra_rot=True):
     """
     NAME:
 
@@ -1139,6 +1191,8 @@ def vxvyvz_to_galcenrect(vx,vy,vz,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
        
        Zsun - Sun's height above the midplane
 
+       _extra_rot= (True) if True, perform an extra tiny rotation to align the Galactocentric coordinate frame with astropy's definition
+
     OUTPUT:
 
        [:,3]= vXg, vYg, vZg
@@ -1149,7 +1203,11 @@ def vxvyvz_to_galcenrect(vx,vy,vz,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
 
        2016-05-12 - Edited to properly take into account the Sun's vertical position; dropped Ysun keyword - Bovy (UofT)
 
+       2018-04-18 - Tweaked to be consistent with astropy's Galactocentric frame - Bovy (UofT)
+
     """
+    if _extra_rot:
+        vx,vy,vz= nu.dot(galcen_extra_rot,nu.array([vx,vy,vz]))
     dgc= nu.sqrt(Xsun**2.+Zsun**2.)
     costheta, sintheta= Xsun/dgc, Zsun/dgc
     return nu.dot(nu.array([[costheta,0.,-sintheta],
@@ -1159,7 +1217,7 @@ def vxvyvz_to_galcenrect(vx,vy,vz,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
 
 @scalarDecorator
 def vxvyvz_to_galcencyl(vx,vy,vz,X,Y,Z,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.,
-                        galcen=False):
+                        galcen=False,_extra_rot=True):
     """
     NAME:
 
@@ -1191,6 +1249,8 @@ def vxvyvz_to_galcencyl(vx,vy,vz,X,Y,Z,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.,
 
        galcen - if True, then X,Y,Z are in cylindrical Galactocentric coordinates rather than rectangular coordinates
 
+       _extra_rot= (True) if True, perform an extra tiny rotation to align the Galactocentric coordinate frame with astropy's definition
+
     OUTPUT:
 
        vRg, vTg, vZg
@@ -1200,12 +1260,14 @@ def vxvyvz_to_galcencyl(vx,vy,vz,X,Y,Z,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.,
        2010-09-24 - Written - Bovy (NYU)
 
     """
-    vxyz= vxvyvz_to_galcenrect(vx,vy,vz,vsun=vsun,Xsun=Xsun,Zsun=Zsun)
+    vxyz= vxvyvz_to_galcenrect(vx,vy,vz,vsun=vsun,Xsun=Xsun,Zsun=Zsun,
+                               _extra_rot=_extra_rot)
     return nu.array(\
         rect_to_cyl_vec(vxyz[:,0],vxyz[:,1],vxyz[:,2],X,Y,Z,cyl=galcen)).T
 
 @scalarDecorator
-def galcenrect_to_vxvyvz(vXg,vYg,vZg,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
+def galcenrect_to_vxvyvz(vXg,vYg,vZg,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.,
+                         _extra_rot=True):
     """
     NAME:
 
@@ -1223,11 +1285,13 @@ def galcenrect_to_vxvyvz(vXg,vYg,vZg,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
 
        vZg - Galactocentric z-velocity
 
-       vsun - velocity of the sun in the GC frame ndarray[3]
+       vsun - velocity of the sun in the GC frame ndarray[3] (can be array of same length as vXg; shape [3,N])
 
-       Xsun - cylindrical distance to the GC
+       Xsun - cylindrical distance to the GC (can be array of same length as vXg)
        
-       Zsun - Sun's height above the midplane
+       Zsun - Sun's height above the midplane (can be array of same length as vXg)
+
+       _extra_rot= (True) if True, perform an extra tiny rotation to align the Galactocentric coordinate frame with astropy's definition
 
     OUTPUT:
 
@@ -1239,17 +1303,38 @@ def galcenrect_to_vxvyvz(vXg,vYg,vZg,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
 
        2016-05-12 - Edited to properly take into account the Sun's vertical position; dropped Ysun keyword - Bovy (UofT)
 
+       2017-10-24 - Allowed vsun/Xsun/Zsun to be arrays - Bovy (UofT)
+
+       2018-04-18 - Tweaked to be consistent with astropy's Galactocentric frame - Bovy (UofT)
+
     """
     dgc= nu.sqrt(Xsun**2.+Zsun**2.)
     costheta, sintheta= Xsun/dgc, Zsun/dgc
-    return nu.dot(nu.array([[-costheta,0.,-sintheta],
-                            [0.,1.,0.],
-                            [-nu.sign(Xsun)*sintheta,0.,
-                              nu.sign(Xsun)*costheta]]),
-                  nu.array([vXg-vsun[0],vYg-vsun[1],vZg-vsun[2]])).T
+    if isinstance(Xsun,nu.ndarray):
+        zero= nu.zeros(len(Xsun))
+        one= nu.ones(len(Xsun))
+        Carr= nu.rollaxis(nu.array([[-costheta,zero,-sintheta],
+                                    [zero,one,zero],
+                                    [-nu.sign(Xsun)*sintheta,zero,
+                                      nu.sign(Xsun)*costheta]]),2)
+        out= ((Carr
+               *nu.array([[vXg-vsun[0],vXg-vsun[0],vXg-vsun[0]],
+                          [vYg-vsun[1],vYg-vsun[1],vYg-vsun[1]],
+                          [vZg-vsun[2],vZg-vsun[2],vZg-vsun[2]]]).T).sum(-1))
+    else:
+        out= nu.dot(nu.array([[-costheta,0.,-sintheta],
+                              [0.,1.,0.],
+                              [-nu.sign(Xsun)*sintheta,0.,
+                                nu.sign(Xsun)*costheta]]),
+                    nu.array([vXg-vsun[0],vYg-vsun[1],vZg-vsun[2]])).T
+    if _extra_rot:
+        return nu.dot(galcen_extra_rot.T,out.T).T
+    else:
+        return out
 
 @scalarDecorator
-def galcencyl_to_vxvyvz(vR,vT,vZ,phi,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
+def galcencyl_to_vxvyvz(vR,vT,vZ,phi,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.,
+                        _extra_rot=True):
     """
     NAME:
 
@@ -1269,11 +1354,13 @@ def galcencyl_to_vxvyvz(vR,vT,vZ,phi,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
 
        phi - Galactocentric azimuth
 
-       vsun - velocity of the sun in the GC frame ndarray[3]
+       vsun - velocity of the sun in the GC frame ndarray[3] (can be array of same length as vRg; shape [3,N])
 
-       Xsun - cylindrical distance to the GC
+       Xsun - cylindrical distance to the GC (can be array of same length as vRg)
        
-       Zsun - Sun's height above the midplane
+       Zsun - Sun's height above the midplane (can be array of same length as vRg)
+
+       _extra_rot= (True) if True, perform an extra tiny rotation to align the Galactocentric coordinate frame with astropy's definition
 
     OUTPUT:
 
@@ -1283,9 +1370,12 @@ def galcencyl_to_vxvyvz(vR,vT,vZ,phi,vsun=[0.,1.,0.],Xsun=1.,Zsun=0.):
 
        2011-02-24 - Written - Bovy (NYU)
 
+       2017-10-24 - Allowed vsun/Xsun/Zsun to be arrays - Bovy (NYU)
+
     """
     vXg, vYg, vZg= cyl_to_rect_vec(vR,vT,vZ,phi)
-    return galcenrect_to_vxvyvz(vXg,vYg,vZg,vsun=vsun,Xsun=Xsun,Zsun=Zsun)
+    return galcenrect_to_vxvyvz(vXg,vYg,vZg,vsun=vsun,Xsun=Xsun,Zsun=Zsun,
+                                _extra_rot=_extra_rot)
 
 def rect_to_cyl_vec(vx,vy,vz,X,Y,Z,cyl=False):
     """
@@ -1666,7 +1756,7 @@ def rphi_to_dl_2d(R,phi,degree=False,ro=1.,phio=0.):
     else:
         return (d,l)
 
-def Rz_to_coshucosv(R,z,delta=1.):
+def Rz_to_coshucosv(R,z,delta=1.,oblate=False):
     """
     NAME:
 
@@ -1684,6 +1774,7 @@ def Rz_to_coshucosv(R,z,delta=1.):
 
        delta= focus
 
+       oblate= (False) if True, compute oblate confocal coordinates instead of prolate
     OUTPUT:
 
        (cosh(u),cos(v))
@@ -1692,14 +1783,22 @@ def Rz_to_coshucosv(R,z,delta=1.):
 
        2012-11-27 - Written - Bovy (IAS)
 
+       2017-10-11 - Added oblate coordinates - Bovy (UofT)
+
     """
-    d12= (z+delta)**2.+R**2.
-    d22= (z-delta)**2.+R**2.
+    if oblate:
+        d12= (R+delta)**2.+z**2.
+        d22= (R-delta)**2.+z**2.
+    else:
+        d12= (z+delta)**2.+R**2.
+        d22= (z-delta)**2.+R**2.
     coshu= 0.5/delta*(sc.sqrt(d12)+sc.sqrt(d22))
     cosv=  0.5/delta*(sc.sqrt(d12)-sc.sqrt(d22))
+    if oblate: # cosv is currently really sinv
+        cosv= sc.sqrt(1.-cosv**2.)
     return (coshu,cosv)
 
-def Rz_to_uv(R,z,delta=1.):
+def Rz_to_uv(R,z,delta=1.,oblate=False):
     """
     NAME:
 
@@ -1707,7 +1806,7 @@ def Rz_to_uv(R,z,delta=1.):
 
     PURPOSE:
 
-       calculate prolate confocal u and v coordinates from R,z, and delta
+       calculate prolate or oblate confocal u and v coordinates from R,z, and delta
 
     INPUT:
 
@@ -1717,6 +1816,8 @@ def Rz_to_uv(R,z,delta=1.):
 
        delta= focus
 
+       oblate= (False) if True, compute oblate confocal coordinates instead of prolate
+
     OUTPUT:
 
        (u,v)
@@ -1725,13 +1826,15 @@ def Rz_to_uv(R,z,delta=1.):
 
        2012-11-27 - Written - Bovy (IAS)
 
+       2017-10-11 - Added oblate coordinates - Bovy (UofT)
+
     """
-    coshu, cosv= Rz_to_coshucosv(R,z,delta)
+    coshu, cosv= Rz_to_coshucosv(R,z,delta,oblate=oblate)
     u= sc.arccosh(coshu)
     v= sc.arccos(cosv)
     return (u,v)
 
-def uv_to_Rz(u,v,delta=1.):
+def uv_to_Rz(u,v,delta=1.,oblate=False):
     """
     NAME:
 
@@ -1749,6 +1852,8 @@ def uv_to_Rz(u,v,delta=1.):
 
        delta= focus
 
+       oblate= (False) if True, compute oblate confocal coordinates instead of prolate
+
     OUTPUT:
 
        (R,z)
@@ -1757,11 +1862,108 @@ def uv_to_Rz(u,v,delta=1.):
 
        2012-11-27 - Written - Bovy (IAS)
 
+       2017-10-11 - Added oblate coordinates - Bovy (UofT)
+
     """
-    R= delta*sc.sinh(u)*sc.sin(v)
-    z= delta*sc.cosh(u)*sc.cos(v)
+    if oblate:
+        R= delta*sc.cosh(u)*sc.sin(v)
+        z= delta*sc.sinh(u)*sc.cos(v)
+    else:
+        R= delta*sc.sinh(u)*sc.sin(v)
+        z= delta*sc.cosh(u)*sc.cos(v)
     return (R,z)
 
+def vRvz_to_pupv(vR,vz,R,z,delta=1.,oblate=False,uv=False):
+    """
+    NAME:
+
+       vRvz_to_pupv
+
+    PURPOSE:
+
+       calculate momenta in prolate or oblate confocal u and v coordinates from cylindrical velocities vR,vz for a given focal length delta
+
+    INPUT:
+
+       vR - radial velocity in cylindrical coordinates
+
+       vz - vertical velocity in cylindrical coordinates
+
+       R - radius
+
+       z - height
+
+       delta= focus
+
+       oblate= (False) if True, compute oblate confocal coordinates instead of prolate
+
+       uv= (False) if True, the given R,z are actually u,v
+
+    OUTPUT:
+
+       (pu,pv)
+
+    HISTORY:
+
+       2017-11-28 - Written - Bovy (UofT)
+
+    """
+    if not uv:
+        u,v= Rz_to_uv(R,z,delta,oblate=oblate)
+    else:
+        u,v= R,z
+    if oblate:
+        pu= delta*(vR*sc.sinh(u)*sc.sin(v)+vz*sc.cosh(u)*sc.cos(v))
+        pv= delta*(vR*sc.cosh(u)*sc.cos(v)-vz*sc.sinh(u)*sc.sin(v))
+    else:
+        pu= delta*(vR*sc.cosh(u)*sc.sin(v)+vz*sc.sinh(u)*sc.cos(v))
+        pv= delta*(vR*sc.sinh(u)*sc.cos(v)-vz*sc.cosh(u)*sc.sin(v))
+    return (pu,pv)
+
+def pupv_to_vRvz(pu,pv,u,v,delta=1.,oblate=False):
+    """
+    NAME:
+
+       pupv_to_vRvz
+
+    PURPOSE:
+
+       calculate cylindrical vR and vz from momenta in prolate or oblate confocal u and v coordinates for a given focal length delta
+
+    INPUT:
+
+       pu - u momentum
+
+       pv - v momentum
+
+       u - u coordinate
+
+       v - v coordinate
+
+       delta= focus
+
+       oblate= (False) if True, compute oblate confocal coordinates instead of prolate
+
+
+    OUTPUT:
+
+       (vR,vz)
+
+    HISTORY:
+
+       2017-12-04 - Written - Bovy (UofT)
+
+    """
+    if oblate:
+        denom= delta*(sc.sinh(u)**2.+sc.cos(v)**2.)
+        vR= (pu*sc.sinh(u)*sc.sin(v)+pv*sc.cosh(u)*sc.cos(v))/denom
+        vz= (pu*sc.cosh(u)*sc.cos(v)-pv*sc.sinh(u)*sc.sin(v))/denom
+    else:
+        denom= delta*(sc.sinh(u)**2.+sc.sin(v)**2.)
+        vR= (pu*sc.cosh(u)*sc.sin(v)+pv*sc.sinh(u)*sc.cos(v))/denom
+        vz= (pu*sc.sinh(u)*sc.cos(v)-pv*sc.cosh(u)*sc.sin(v))/denom
+    return (vR,vz)
+
 def Rz_to_lambdanu(R,z,ac=5.,Delta=1.):
     """
     NAME:
@@ -1998,6 +2200,66 @@ def radec_to_custom(ra,dec,T=None,degree=False,epoch=2000.0):
     out= nu.array([l,b])
     return out.T   
 
+@scalarDecorator
+@degreeDecorator([2,3],[])
+def pmrapmdec_to_custom(pmra,pmdec,ra,dec,T=None,degree=False,epoch=2000.0):
+    """
+    NAME:
+
+       pmrapmdec_to_custom
+
+    PURPOSE:
+
+       rotate proper motions in (ra,dec) to proper motions in a custom set of sky coordinates (phi1,phi2)
+
+    INPUT:
+
+       pmra - proper motion in ra (multplied with cos(dec)) [mas/yr]
+
+       pmdec - proper motion in dec [mas/yr]
+
+       ra - right ascension
+
+       dec - declination
+
+       T= matrix defining the transformation: new_rect= T dot old_rect, where old_rect = [cos(dec)cos(ra),cos(dec)sin(ra),sin(dec)] and similar for new_rect
+
+       degree= (False) if True, ra and dec are given in degrees (default=False)
+
+       epoch= (2000.) epoch of ra,dec (right now only 2000.0 and 1950.0 are supported when not using astropy's transformations internally; when internally using astropy's coordinate transformations, epoch can be None for ICRS, 'JXXXX' for FK5, and 'BXXXX' for FK4)
+
+    OUTPUT:
+
+       (pmphi1 x cos(phi2),pmph2) for vector inputs [:,2]
+
+    HISTORY:
+
+       2016-10-24 - Written - Bovy (UofT/CCA)
+
+    """
+    if T is None: raise ValueError("Must set T= for radec_to_custom")
+    # Need to figure out ra_ngp and dec_ngp for this custom set of sky coords
+    # Should replace this eventually with custom_to_radec
+    customXYZ= nu.dot(T.T,nu.array([0.,0.,1.]))
+    dec_ngp= nu.arcsin(customXYZ[2])
+    ra_ngp= nu.arctan2(customXYZ[1]/sc.cos(dec_ngp),
+                       customXYZ[0]/sc.cos(dec_ngp))
+    #Whether to use degrees and scalar input is handled by decorators
+    dec[dec == dec_ngp]+= 10.**-16 #deal w/ pole.
+    sindec_ngp= nu.sin(dec_ngp)
+    cosdec_ngp= nu.cos(dec_ngp)
+    sindec= nu.sin(dec)
+    cosdec= nu.cos(dec)
+    sinrarangp= nu.sin(ra-ra_ngp)
+    cosrarangp= nu.cos(ra-ra_ngp)
+    cosphi= sindec_ngp*cosdec-cosdec_ngp*sindec*cosrarangp
+    sinphi= sinrarangp*cosdec_ngp
+    norm= nu.sqrt(cosphi**2.+sinphi**2.)
+    cosphi/= norm
+    sinphi/= norm
+    return (nu.array([[cosphi,-sinphi],[sinphi,cosphi]]).T\
+                *nu.array([[pmra,pmra],[pmdec,pmdec]]).T).sum(-1)
+
 def get_epoch_angles(epoch=2000.0):
     """
     NAME:
@@ -2009,7 +2271,7 @@ def get_epoch_angles(epoch=2000.0):
        get the angles relevant for the transformation from ra, dec to l,b for the given epoch
     INPUT:
 
-       epoch - epoch of ra,dec (right now only 2000.0 and 1950.0 are supported when not using astropy's transformations internally; when internally using astropy's coordinate transformations, epoch can be None for ICRS, 'JXXXX' for FK5, and 'BXXXX' for FK4)
+       epoch - epoch of ra,dec (right now only 2000.0 and 1950.0 are supported when not using astropy's transformations internally; when internally using astropy's coordinate transformations, epoch can be None for ICRS, 'JXXXX' for FK5, and 'BXXXX' for FK4 [but for B1950 FK4 with no E abberation terms is assumed... really, there's no reason to use B1950 in 2018 when using galpy...))
 
     OUTPUT:
 
@@ -2021,15 +2283,22 @@ def get_epoch_angles(epoch=2000.0):
 
        2016-05-13 - Added support for using astropy's coordinate transformations and for non-standard epochs - Bovy (UofT)
 
+       2018-04-18 - Edited J2000 angles to be fully consistent with astropy - BOvy (UofT)
+
     """
     if epoch == 2000.0:
-        theta= 122.932/180.*sc.pi
-        dec_ngp= 27.12825/180.*sc.pi
-        ra_ngp= 192.85948/180.*sc.pi
+        # Following astropy's definition here
+        theta= 122.9319185680026/180.*sc.pi
+        dec_ngp= 27.12825118085622/180.*sc.pi
+        ra_ngp= 192.8594812065348/180.*sc.pi
     elif epoch == 1950.0:
         theta= 123./180.*sc.pi
         dec_ngp= 27.4/180.*sc.pi
         ra_ngp= 192.25/180.*sc.pi
+    elif epoch == None: # obtained below
+        theta= theta_icrs
+        dec_ngp= dec_ngp_icrs
+        ra_ngp= ra_ngp_icrs
     elif _APY_LOADED:
         # Use astropy to get the angles
         epoch, frame= _parse_epoch_frame_apy(epoch)
@@ -2043,14 +2312,29 @@ def get_epoch_angles(epoch=2000.0):
             c= c.transform_to(apycoords.FK5(equinox=epoch))
         elif not epoch is None and 'B' in epoch:
             c= c.transform_to(apycoords.FK4(equinox=epoch))
-        else:
-            c= c.transform_to(apycoords.ICRS)
+        else: # pragma: no cover
+            raise ValueError('epoch input not understood; should be None for ICRS, JXXXX, or BXXXX')
         dec_ngp= c.dec.to(units.rad).value
         ra_ngp= c.ra.to(units.rad).value
     else:
         raise IOError("Only epochs 1950 and 2000 are supported if you don't have astropy")
     return (theta,dec_ngp,ra_ngp)
 
+# Get ICRS angles once when astropy is installed
+if _APY_LOADED:
+    c= apycoords.SkyCoord(180.*units.deg,90.*units.deg,frame='icrs')
+    c= c.transform_to(apycoords.Galactic)
+    theta_icrs= c.l.to(units.rad).value
+    c= apycoords.SkyCoord(180.*units.deg,90.*units.deg,
+                          frame='galactic')
+    c= c.transform_to(apycoords.ICRS)
+    dec_ngp_icrs= c.dec.to(units.rad).value
+    ra_ngp_icrs= c.ra.to(units.rad).value
+else:
+    theta_icrs= 2.1455668515225916
+    dec_ngp_icrs= 0.4734773249532947
+    ra_ngp_icrs= 3.366032882941063
+
 def _parse_epoch_frame_apy(epoch):
     if epoch == 2000.0 or epoch == '2000': epoch= 'J2000'
     elif epoch == 1950.0 or epoch == '1950': epoch= 'B1950'
@@ -2058,3 +2342,42 @@ def _parse_epoch_frame_apy(epoch):
     elif not epoch is None and 'B' in epoch: frame= 'fk4'
     else: frame= 'icrs'
     return (epoch,frame)
+
+# Matrix to rotate to the astropy Galactocentric frame: astropy's 
+# Galactocentric frame is slightly off from the one that we get by simply 
+# taking Galactic coordinates and transforming them: transformation from
+# Bovy (2011) maps NGP --> (0,0,1), astropy to (v. small, v. small, 1-v. small)
+# so we rotate Bovy (2011) such that we agree; for that we compute what NGP 
+# goes to using astropy's transformations
+theta,dec_ngp,ra_ngp= get_epoch_angles(None) # None = ICRS, basis for astropy
+dec_gc,ra_gc= -28.936175/180.*nu.pi,266.4051/180.*nu.pi # from apy def.
+eta= 58.5986320306/180.*nu.pi # astropy 'roll' angle
+gc_vec= nu.array(\
+    [nu.cos(theta)*(-nu.sin(dec_ngp)*nu.cos(dec_gc)*nu.cos(ra_gc-ra_ngp)
+                     +nu.cos(dec_ngp)*nu.sin(dec_gc))
+     +nu.sin(theta)*nu.cos(dec_gc)*nu.sin(ra_gc-ra_ngp),
+     nu.sin(theta)*(-nu.sin(dec_ngp)*nu.cos(dec_gc)*nu.cos(ra_gc-ra_ngp)
+                     +nu.cos(dec_ngp)*nu.sin(dec_gc))
+    -nu.cos(theta)*nu.cos(dec_gc)*nu.sin(ra_gc-ra_ngp),
+     nu.cos(dec_ngp)*nu.cos(dec_gc)*nu.cos(ra_gc-ra_ngp)
+     +nu.sin(dec_ngp)*nu.sin(dec_gc)])
+galcen_extra_rot1= _rotate_to_arbitrary_vector(nu.atleast_2d(gc_vec),
+                                               nu.array([1.,0.,0.]),
+                                               inv=False,_dontcutsmall=True)[0]
+ngp_vec= nu.dot(galcen_extra_rot1,nu.array(\
+    [-nu.cos(dec_gc)*nu.cos(dec_ngp)*nu.cos(ra_ngp-ra_gc)
+      -nu.sin(dec_gc)*nu.sin(dec_ngp),
+      nu.cos(eta)*nu.cos(dec_ngp)*nu.sin(ra_ngp-ra_gc)
+      +nu.sin(eta)*(-nu.sin(dec_gc)*nu.cos(dec_ngp)*nu.cos(ra_ngp-ra_gc)
+                     +nu.cos(dec_gc)*nu.sin(dec_ngp)),
+      -nu.sin(eta)*nu.cos(dec_ngp)*nu.sin(ra_ngp-ra_gc)
+      +nu.cos(eta)*(-nu.sin(dec_gc)*nu.cos(dec_ngp)*nu.cos(ra_ngp-ra_gc)
+                     +nu.cos(dec_gc)*nu.sin(dec_ngp))]))
+galcen_extra_rot2= _rotate_to_arbitrary_vector(nu.atleast_2d(ngp_vec),
+                                               nu.array([0.,0.,1.]),
+                                               inv=True,_dontcutsmall=True)[0]
+# Leave x axis alone, because in place by rot1
+galcen_extra_rot2[0,0]= 1.
+galcen_extra_rot2[0,1:]= 0.
+galcen_extra_rot2[1:,0]= 0.
+galcen_extra_rot= nu.dot(galcen_extra_rot2,galcen_extra_rot1)
